﻿using Kivii.Finances.Icbc.Entities;
using Kivii.Icbc.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Xml;

namespace Kivii.Finances.Icbc
{
    internal static class Extension
    {
        private static List<string> _icbcXML { get; set; }

        /// <summary>
        /// 获取ICBC流水
        /// </summary>
        /// <param name="nc1">netSafeClient</param>
        /// <param name="icbc">ICBCConfig</param>
        /// <param name="nextTag">分页标识,传String.Empty</param>
        /// <param name="beginTime">查询开始时间</param>
        /// <param name="endTime">结束时间</param>
        /// <param name="newGet">查询开始标识,只可传入True</param>
        /// <returns></returns>
        internal static List<string> GetICBCDetail(this ICBC_NetSafeClient nc1, ICBCConfig icbc, string nextTag, DateTime beginTime, DateTime endTime, bool newGet, out string balance)
        {
            if (newGet)
            {
                _icbcXML = new List<string>();
                nextTag = string.Empty;
            }
            XmlDocument QACCBAL = new XmlDocument();
            //加载Assembly的资源文件
            QACCBAL.Load(Assembly.GetExecutingAssembly().GetManifestResourceStream("Kivii.Finances.Icbc.Content.QACCBAL.txt"));
            XML_EASY xML_EASY = new XML_EASY(QACCBAL.InnerXml);
            xML_EASY.SetXMLNode("AccNo", icbc.AccNo);
            nc1.Run(xML_EASY.GetXML(), out string seqNo, out string result);
            xML_EASY.SetXML(result);
            if (xML_EASY.LocateString("<error>") >= 0)
            {
                Console.WriteLine(xML_EASY.GetXMLNode("error"));
                balance = "error";
                return new List<string>
                {
                    "error"
                };
            }
            string xMLNode = xML_EASY.GetXMLNode("Balance");
            if (xMLNode != null && xMLNode.Length > 0)
            {
                double num = Convert.ToDouble(xMLNode) / 100.0;
                string xMLNode2 = xML_EASY.GetXMLNode("QueryTime");
                string xMLNode3 = xML_EASY.GetXMLNode("AccName");
                Console.WriteLine("账号户名：" + xMLNode3 + "\r\n账号余额：" + num.ToString() + "\r\n银行主机系统时间：" + xMLNode2, "银行返回信息" + seqNo);
                balance = num.ToString();
            }
            else
            {
                Console.WriteLine("RetMsg：\r\n" + xML_EASY.GetXMLNode("RetMsg"), "银行返回信息" + seqNo);
                balance = null;
            }
            XmlDocument QHISD = new XmlDocument();
            QHISD.Load(Assembly.GetExecutingAssembly().GetManifestResourceStream("Kivii.Finances.Icbc.Content.QHISD.txt"));
            xML_EASY = new XML_EASY(QHISD.InnerXml);
            xML_EASY.SetXMLNode("AccNo", icbc.AccNo);
            xML_EASY.SetXMLNode("BeginDate", beginTime.ToString("yyyyMMdd"));
            xML_EASY.SetXMLNode("EndDate", endTime.ToString("yyyyMMdd"));
            xML_EASY.SetXMLNode("MinAmt", icbc.MinAmt);
            xML_EASY.SetXMLNode("MaxAmt", icbc.MaxAmt);
            xML_EASY.SetXMLNode("NextTag", nextTag);
            nc1.Run(xML_EASY.GetXML(), out string seqNo2, out string rtn);

            if (_icbcXML == null) _icbcXML = new List<string>();
            _icbcXML.Add(rtn);

            XmlDocument doc = new XmlDocument();
            doc.LoadXml(rtn);
            var tag = doc.GetElementsByTagName("NextTag")[0].InnerXml;
            if (!tag.IsNullOrEmpty())
            {
                nc1.GetICBCDetail(icbc, tag, beginTime, endTime, false, out string currentBalance);
            }
            return _icbcXML;
        }

        internal static List<ICBC> GetICBCDetail(this ICBC_NetSafeClient nc1, ICBCConfig icbc, string nextTag, DateTime beginTime, DateTime endTime)
        {
            //当首次进行查询  把余额也查出来便于和系统余额比对
            if (nextTag.IsNullOrEmpty())
            {
                XmlDocument QACCBAL = new XmlDocument();
                QACCBAL.Load(Assembly.GetExecutingAssembly().GetManifestResourceStream("Kivii.Finances.Icbc.Content.QACCBAL.txt"));
                XML_EASY xmlEasy = new XML_EASY(QACCBAL.InnerXml);
                xmlEasy.SetXMLNode("AccNo", icbc.AccNo);
                nc1.Run(xmlEasy.GetXML(), out string seqNo, out string xmlResult);
                xmlEasy.SetXML(xmlResult);
                if (xmlEasy.LocateString("<error>") >= 0)
                {
                    Console.WriteLine(xmlEasy.GetXMLNode("error"));
                    icbc.currentBalance = "error";
                }
                string xMLNode = xmlEasy.GetXMLNode("Balance");
                if (xMLNode != null && xMLNode.Length > 0)
                {
                    double num = Convert.ToDouble(xMLNode) / 100.0;
                    string xMLNode2 = xmlEasy.GetXMLNode("QueryTime");
                    string xMLNode3 = xmlEasy.GetXMLNode("AccName");
                    Console.WriteLine("账号户名：" + xMLNode3 + "\r\n账号余额：" + num.ToString() + "\r\n银行主机系统时间：" + xMLNode2, "银行返回信息" + seqNo);
                    icbc.currentBalance = num.ToString();
                }
            }
            XmlDocument QHISD = new XmlDocument();
            QHISD.Load(Assembly.GetExecutingAssembly().GetManifestResourceStream("Kivii.Finances.Icbc.Content.QHISD.txt"));
            var xML_EASY = new XML_EASY(QHISD.InnerXml);
            xML_EASY.SetXMLNode("AccNo", icbc.AccNo);
            xML_EASY.SetXMLNode("BeginDate", beginTime.ToString("yyyyMMdd"));
            xML_EASY.SetXMLNode("EndDate", endTime.ToString("yyyyMMdd"));
            xML_EASY.SetXMLNode("MinAmt", icbc.MinAmt);
            xML_EASY.SetXMLNode("MaxAmt", icbc.MaxAmt);
            xML_EASY.SetXMLNode("NextTag", nextTag);
            nc1.Run(xML_EASY.GetXML(), out string seqNo2, out string result);

            xML_EASY.SetXML(result);
            if (xML_EASY.LocateString("<error>") >= 0) throw new Exception("查询失败");

            XmlDocument doc = new XmlDocument();
            doc.LoadXml(result);
            doc.ThrowIfNull("暂未获取到银行流水数据");

            XmlNodeList xmlNodeList = doc.GetElementsByTagName("rd");
            var rtns = xmlNodeList.XmlToObject();

            var accName = doc.GetElementsByTagName("AccName")[0].InnerText;
            var accNo = doc.GetElementsByTagName("AccNo")[0].InnerText;
            icbc.AccName = accName;
            var tag = doc.GetElementsByTagName("NextTag")[0].InnerXml;
            if (!tag.IsNullOrEmpty())
            {
                var more = nc1.GetICBCDetail(icbc, tag, beginTime, endTime);
                rtns.AddRange(more);
                return rtns;
            }

            return rtns;
        }

        internal static string GetICBCBalance(this ICBC_NetSafeClient nc1, ICBCConfig icbc)
        {
            XmlDocument QACCBAL = new XmlDocument();
            //加载Assembly的资源文件
            QACCBAL.Load(Assembly.GetExecutingAssembly().GetManifestResourceStream("Kivii.Finances.Icbc.Content.QACCBAL.txt"));
            XML_EASY xML_EASY = new XML_EASY(QACCBAL.InnerXml);
            xML_EASY.SetXMLNode("AccNo", icbc.AccNo);
            nc1.Run(xML_EASY.GetXML(), out string seqNo, out string result);
            xML_EASY.SetXML(result);
            if (xML_EASY.LocateString("<error>") >= 0)
            {
                Console.WriteLine(xML_EASY.GetXMLNode("error"));
                return "error";
            }

            string balance;

            string xMLNode = xML_EASY.GetXMLNode("Balance");
            if (xMLNode != null && xMLNode.Length > 0)
            {
                double num = Convert.ToDouble(xMLNode) / 100.0;
                string xMLNode2 = xML_EASY.GetXMLNode("QueryTime");
                string xMLNode3 = xML_EASY.GetXMLNode("AccName");
                Console.WriteLine("账号户名：" + xMLNode3 + "\r\n账号余额：" + num.ToString() + "\r\n银行主机系统时间：" + xMLNode2, "银行返回信息" + seqNo);
                balance = num.ToString();
            }
            else
            {
                Console.WriteLine("RetMsg：\r\n" + xML_EASY.GetXMLNode("RetMsg"), "银行返回信息" + seqNo);
                balance = null;
            }

            return balance;
        }

        /// <summary>
        /// 将xml转成ICBC实体对象
        /// </summary>
        /// <param name="xmlNodeList"></param>
        /// <returns></returns>
        internal static List<ICBC> XmlToObject(this XmlNodeList xmlNodeList)
        {
            List<ICBC> rtns = new List<ICBC>();
            var properties = typeof(ICBC).GetProperties();
            try
            {
                foreach (XmlElement item in xmlNodeList)
                {
                    var entity = new ICBC();
                    foreach (XmlElement node in item)
                    {
                        var property = properties.FirstOrDefault(o => o.Name == node.Name);
                        if (property == null) continue;
                        if (!property.PropertyType.IsGenericType)
                        {
                            //非泛型
                            property.SetValue(entity, string.IsNullOrEmpty(node.InnerText) ? null : Convert.ChangeType(node.InnerText, property.PropertyType), null);
                        }
                        else
                        {
                            //泛型Nullable<>
                            Type genericTypeDefinition = property.PropertyType.GetGenericTypeDefinition();
                            if (genericTypeDefinition == typeof(Nullable<>))
                            {
                                property.SetValue(entity, string.IsNullOrEmpty(node.InnerText) ? null : Convert.ChangeType(node.InnerText, Nullable.GetUnderlyingType(property.PropertyType)), null);
                            }
                        }
                    }
                    rtns.Add(entity);
                }
            }
            catch (Exception ex)
            {
                return rtns;
            }
            return rtns;
        }
    }
}
